@@ -24,7 +24,6 @@
 
 #include <config.h>
 #include <stdlib.h>
-#include <gtk/gtksignal.h>
 #include "gal/util/e-util.h"
 #include "e-table-subset.h"
 
@@ -36,12 +35,14 @@ static void etss_proxy_model_cell_changed_real (ETableSubset *etss, ETableModel
 static void etss_proxy_model_rows_inserted_real (ETableSubset *etss, ETableModel *etm, int row, int count);
 static void etss_proxy_model_rows_deleted_real (ETableSubset *etss, ETableModel *etm, int row, int count);
 
-#define PARENT_TYPE E_TABLE_MODEL_TYPE
 #define d(x)
 
 static ETableModelClass *etss_parent_class;
 
-#define ETSS_CLASS(object) (E_TABLE_SUBSET_CLASS(GTK_OBJECT_GET_CLASS(object)))
+#define ETSS_CLASS(object) (E_TABLE_SUBSET_GET_CLASS(object))
+
+#define VALID_ROW(etss, row) (row >= -1 && row < etss->n_map)
+#define MAP_ROW(etss, row) (row == -1 ? -1 : etss->map_table[row])
 
 static gint
 etss_get_view_row (ETableSubset *etss, int row)
@@ -81,27 +82,27 @@ etss_get_view_row (ETableSubset *etss, int row)
 }
 
 static void
-etss_destroy (GtkObject *object)
+etss_dispose (GObject *object)
 {
 	ETableSubset *etss = E_TABLE_SUBSET (object);
 
 	if (etss->source) {
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_pre_change_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_no_change_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_changed_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_row_changed_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_cell_changed_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_rows_inserted_id);
-		gtk_signal_disconnect (GTK_OBJECT (etss->source),
-				       etss->table_model_rows_deleted_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_pre_change_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_no_change_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_changed_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_row_changed_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_cell_changed_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_rows_inserted_id);
+		g_signal_handler_disconnect (G_OBJECT (etss->source),
+					     etss->table_model_rows_deleted_id);
 
-		gtk_object_unref (GTK_OBJECT (etss->source));
+		g_object_unref (etss->source);
 		etss->source = NULL;
 
 		etss->table_model_changed_id = 0;
@@ -111,10 +112,18 @@ etss_destroy (GtkObject *object)
 		etss->table_model_rows_deleted_id = 0;
 	}
 
+	G_OBJECT_CLASS (etss_parent_class)->dispose (object);
+}
+
+static void
+etss_finalize (GObject *object)
+{
+	ETableSubset *etss = E_TABLE_SUBSET (object);
+
 	g_free (etss->map_table);
 	etss->map_table = NULL;
 
-	GTK_OBJECT_CLASS (etss_parent_class)->destroy (object);
+	G_OBJECT_CLASS (etss_parent_class)->finalize (object);
 }
 
 static int
@@ -138,9 +147,11 @@ etss_value_at (ETableModel *etm, int col, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
+	g_return_val_if_fail (VALID_ROW (etss, row), NULL);
+
 	etss->last_access = row;
 	d(g_print("g) Setting last_access to %d\n", row));
-	return e_table_model_value_at (etss->source, col, etss->map_table [row]);
+	return e_table_model_value_at (etss->source, col, MAP_ROW(etss, row));
 }
 
 static void
@@ -148,9 +159,11 @@ etss_set_value_at (ETableModel *etm, int col, int row, const void *val)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
+	g_return_if_fail (VALID_ROW (etss, row));
+
 	etss->last_access = row;
 	d(g_print("h) Setting last_access to %d\n", row));
-	e_table_model_set_value_at (etss->source, col, etss->map_table [row], val);
+	e_table_model_set_value_at (etss->source, col, MAP_ROW(etss, row), val);
 }
 
 static gboolean
@@ -158,7 +171,9 @@ etss_is_cell_editable (ETableModel *etm, int col, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
-	return e_table_model_is_cell_editable (etss->source, col, etss->map_table [row]);
+	g_return_val_if_fail (VALID_ROW (etss, row), FALSE);
+
+	return e_table_model_is_cell_editable (etss->source, col, MAP_ROW(etss, row));
 }
 
 static gboolean
@@ -172,10 +187,12 @@ etss_get_save_id (ETableModel *etm, int row)
 {
 	ETableSubset *etss = (ETableSubset *)etm;
 
+	g_return_val_if_fail (VALID_ROW (etss, row), NULL);
+
 	if (e_table_model_has_save_id (etss->source))
-		return e_table_model_get_save_id (etss->source, etss->map_table [row]);
+		return e_table_model_get_save_id (etss->source, MAP_ROW(etss, row));
 	else
-		return g_strdup_printf ("%d", etss->map_table[row]);
+		return g_strdup_printf ("%d", MAP_ROW(etss, row));
 }
 
 static void
@@ -226,37 +243,38 @@ etss_value_to_string (ETableModel *etm, int col, const void *value)
 }
 
 static void
-etss_class_init (GtkObjectClass *object_class)
+etss_class_init (GObjectClass *object_class)
 {
-	ETableSubsetClass *klass        = (ETableSubsetClass *) object_class;
-	ETableModelClass *table_class   = (ETableModelClass *) object_class;
+	ETableSubsetClass *klass         = (ETableSubsetClass *) object_class;
+	ETableModelClass *table_class    = (ETableModelClass *) object_class;
 
-	etss_parent_class               = gtk_type_class (PARENT_TYPE);
+	etss_parent_class                = g_type_class_peek_parent (klass);
 	
-	object_class->destroy           = etss_destroy;
+	object_class->dispose            = etss_dispose;
+	object_class->finalize           = etss_finalize;
 
-	table_class->column_count       = etss_column_count;
-	table_class->row_count          = etss_row_count;
-	table_class->append_row         = etss_append_row;
+	table_class->column_count        = etss_column_count;
+	table_class->row_count           = etss_row_count;
+	table_class->append_row          = etss_append_row;
 
-	table_class->value_at           = etss_value_at;
-	table_class->set_value_at       = etss_set_value_at;
-	table_class->is_cell_editable   = etss_is_cell_editable;
+	table_class->value_at            = etss_value_at;
+	table_class->set_value_at        = etss_set_value_at;
+	table_class->is_cell_editable    = etss_is_cell_editable;
 
-	table_class->has_save_id        = etss_has_save_id;
-	table_class->get_save_id        = etss_get_save_id;
+	table_class->has_save_id         = etss_has_save_id;
+	table_class->get_save_id         = etss_get_save_id;
 
-	table_class->duplicate_value    = etss_duplicate_value;
-	table_class->free_value         = etss_free_value;
-	table_class->initialize_value   = etss_initialize_value;
-	table_class->value_is_empty     = etss_value_is_empty;
-	table_class->value_to_string    = etss_value_to_string;
+	table_class->duplicate_value     = etss_duplicate_value;
+	table_class->free_value          = etss_free_value;
+	table_class->initialize_value    = etss_initialize_value;
+	table_class->value_is_empty      = etss_value_is_empty;
+	table_class->value_to_string     = etss_value_to_string;
 
-	klass->proxy_model_pre_change   = etss_proxy_model_pre_change_real;
+	klass->proxy_model_pre_change    = etss_proxy_model_pre_change_real;
 	klass->proxy_model_no_change     = etss_proxy_model_no_change_real;
-	klass->proxy_model_changed      = etss_proxy_model_changed_real;
-	klass->proxy_model_row_changed  = etss_proxy_model_row_changed_real;
-	klass->proxy_model_cell_changed = etss_proxy_model_cell_changed_real;
+	klass->proxy_model_changed       = etss_proxy_model_changed_real;
+	klass->proxy_model_row_changed   = etss_proxy_model_row_changed_real;
+	klass->proxy_model_cell_changed  = etss_proxy_model_cell_changed_real;
 	klass->proxy_model_rows_inserted = etss_proxy_model_rows_inserted_real;
 	klass->proxy_model_rows_deleted  = etss_proxy_model_rows_deleted_real;
 }
@@ -267,7 +285,7 @@ etss_init (ETableSubset *etss)
 	etss->last_access = 0;
 }
 
-E_MAKE_TYPE(e_table_subset, "ETableSubset", ETableSubset, etss_class_init, etss_init, PARENT_TYPE)
+E_MAKE_TYPE(e_table_subset, "ETableSubset", ETableSubset, etss_class_init, etss_init, E_TABLE_MODEL_TYPE)
 
 static void
 etss_proxy_model_pre_change_real (ETableSubset *etss, ETableModel *etm)
@@ -383,26 +401,26 @@ e_table_subset_construct (ETableSubset *etss, ETableModel *source, int nvals)
 	etss->map_table = buffer;
 	etss->n_map = nvals;
 	etss->source = source;
-	gtk_object_ref (GTK_OBJECT (source));
+	g_object_ref (source);
 	
 	/* Init */
 	for (i = 0; i < nvals; i++)
 		etss->map_table [i] = i;
 
-	etss->table_model_pre_change_id = gtk_signal_connect (GTK_OBJECT (source), "model_pre_change",
-							      GTK_SIGNAL_FUNC (etss_proxy_model_pre_change), etss);
-	etss->table_model_no_change_id     = gtk_signal_connect (GTK_OBJECT (source), "model_no_change",
-								 GTK_SIGNAL_FUNC (etss_proxy_model_no_change), etss);
-	etss->table_model_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_changed",
-						     GTK_SIGNAL_FUNC (etss_proxy_model_changed), etss);
-	etss->table_model_row_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_row_changed",
-							 GTK_SIGNAL_FUNC (etss_proxy_model_row_changed), etss);
-	etss->table_model_cell_changed_id = gtk_signal_connect (GTK_OBJECT (source), "model_cell_changed",
-							  GTK_SIGNAL_FUNC (etss_proxy_model_cell_changed), etss);
-	etss->table_model_rows_inserted_id = gtk_signal_connect (GTK_OBJECT (source), "model_rows_inserted",
-								GTK_SIGNAL_FUNC (etss_proxy_model_rows_inserted), etss);
-	etss->table_model_rows_deleted_id = gtk_signal_connect (GTK_OBJECT (source), "model_rows_deleted",
-							       GTK_SIGNAL_FUNC (etss_proxy_model_rows_deleted), etss);
+	etss->table_model_pre_change_id    = g_signal_connect (G_OBJECT (source), "model_pre_change",
+							G_CALLBACK (etss_proxy_model_pre_change), etss);
+	etss->table_model_no_change_id     = g_signal_connect (G_OBJECT (source), "model_no_change",
+							G_CALLBACK (etss_proxy_model_no_change), etss);
+	etss->table_model_changed_id       = g_signal_connect (G_OBJECT (source), "model_changed",
+							G_CALLBACK (etss_proxy_model_changed), etss);
+	etss->table_model_row_changed_id   = g_signal_connect (G_OBJECT (source), "model_row_changed",
+							G_CALLBACK (etss_proxy_model_row_changed), etss);
+	etss->table_model_cell_changed_id  = g_signal_connect (G_OBJECT (source), "model_cell_changed",
+							G_CALLBACK (etss_proxy_model_cell_changed), etss);
+	etss->table_model_rows_inserted_id = g_signal_connect (G_OBJECT (source), "model_rows_inserted",
+							G_CALLBACK (etss_proxy_model_rows_inserted), etss);
+	etss->table_model_rows_deleted_id  = g_signal_connect (G_OBJECT (source), "model_rows_deleted",
+							G_CALLBACK (etss_proxy_model_rows_deleted), etss);
 	
 	return E_TABLE_MODEL (etss);
 }
@@ -410,18 +428,19 @@ e_table_subset_construct (ETableSubset *etss, ETableModel *source, int nvals)
 ETableModel *
 e_table_subset_new (ETableModel *source, const int nvals)
 {
-	ETableSubset *etss = gtk_type_new (E_TABLE_SUBSET_TYPE);
+	ETableSubset *etss = g_object_new (E_TABLE_SUBSET_TYPE, NULL);
 
 	if (e_table_subset_construct (etss, source, nvals) == NULL){
-		gtk_object_unref (GTK_OBJECT (etss));
+		g_object_unref (etss);
 		return NULL;
 	}
 
 	return (ETableModel *) etss;
 }
 
-int          e_table_subset_model_to_view_row  (ETableSubset *ets,
-						int           model_row)
+int
+e_table_subset_model_to_view_row  (ETableSubset *ets,
+				   int           model_row)
 {
 	int i;
 	for (i = 0; i < ets->n_map; i++) {
@@ -431,8 +450,9 @@ int          e_table_subset_model_to_view_row  (ETableSubset *ets,
 	return -1;
 }
 
-int          e_table_subset_view_to_model_row  (ETableSubset *ets,
-						int           view_row)
+int
+e_table_subset_view_to_model_row  (ETableSubset *ets,
+				   int           view_row)
 {
 	if (view_row >= 0 && view_row < ets->n_map)
 		return ets->map_table[view_row];
